"use strict";
this.name           = "PlanetEngine"; 
this.author         = "Stranger"; 
this.copyright      = "This script is hereby placed in the public domain."; 
this.version        = "1.7"; 
this.description    = "Gives info and define texture to additional planets genereted by Orbits";

this.$planetRadius = 6400;
this.$planetDensity = 5.525;
this.$planetEscapeVelocity = 11180;
this.$surfaceGravity = 1;
this.$atmosphereMass = 1;

this.$planetFlag = -1;

this.$orbitRadiusList = new Array();
this.$orbitPeriodList = new Array();
this.$insolationList = new Array();
this.$planetDensityList = new Array();
this.$planetMassList = new Array();
this.$escapeVelocityList = new Array();
this.$planetGravityList = new Array();
this.$atmosphereMassList = new Array();
this.$atmospherePressureList = new Array();
this.$atmosphereTemperatureList = new Array();

this.shipWillExitWitchspace = function()
    {
    if (system.isInterstellarSpace) return;
    this.$scriptDelay = new Timer(this, this.$scanSystem, 0.25);    //delay to set planet orbits!
    }

this.shipWillLaunchFromStation = function()
    {
    this.shipWillExitWitchspace();
    delete this.shipWillLaunchFromStation;
    }

this.shipApproachingPlanetSurface = function(planet) 
    {
    this.$planetFlag = planet.radius;
    }

this.shipDockedWithStation = function(station)
    {
    if(station.hasRole("planetFall_surface"))
        {
        this.$initInterface(station);    
        }
    }

this.$initInterface = function(station)
    {
	station.setInterface(this.name,{
		title:"Planet Data Sheet",
		category:"Ambience",
		summary:"Provide extended set of planetary data",
		callback:this.$setupMissionPage.bind(this)
        });
    }

this.$setupMissionPage = function()
    {
    if(this.$planetFlag > 25000)
        {
        this.$planetInfo();
        }
    else
        {
        this.$moonInfo();
        }
    }

this.$scanSystem = function()
    {
    var w = worldScripts.AstroLibrary;
	var info = system.info;
    if (system.info.sun_radius)
		{
        var sunRadius = system.sun.radius;
		}
    var sunMass = w.$astroLib_sunMass(sunRadius);
    var sunTemperature = w.$astroLib_sunTemperature(sunRadius);
    var sunLuminosity = w.$astroLib_sunLuminosity(sunTemperature,sunRadius);
    var localYear = w.$astroLib_solPeriod(sunMass);
    var mainOrbitVector = new Vector3D(system.sun.position.subtract(system.mainPlanet.position));
    var ouScale = mainOrbitVector.magnitude();
    var planetRadius = this.$planetRadius;
    var planetDensity = this.$planetDensity;
    var escapeVelocity = this.$planetEscapeVelocity;
    var surfaceGravity = this.$surfaceGravity;
    var atmosphereMass = this.$atmosphereMass;
    var mainInsolation = sunLuminosity * Math.pow((50000000/ouScale),2);
    var insolation = mainInsolation;
    var meanTemperature = w.$astroLib_equilibriumTemperature(insolation);
    var equilibriumTemperature = meanTemperature;
    this.$orbitRadiusList = w.$astroLib_orbitRadius();
    for (let i=0; i<system.planets.length; i++)
        {
        planetRadius = system.planets[i].radius/10;        
        this.$orbitPeriodList[i] = localYear * Math.pow(this.$orbitRadiusList[i],1.5);
        this.$insolationList[i] = mainInsolation * Math.pow(this.$orbitRadiusList[i],-2);
        insolation = this.$insolationList[i];
        meanTemperature = w.$astroLib_equilibriumTemperature(insolation);
        equilibriumTemperature = meanTemperature;
        if(planetRadius < 10000)
            {
            this.$planetDensityList[i] = w.$astroLib_planetDensity(planetRadius, equilibriumTemperature);
            }
        else
            {
            if(planetRadius < 15000)
                {
                this.$planetDensityList[i] = 0.5 + 1.5 * (15000 - planetRadius)/5000 + 0.25 * system.scrambledPseudoRandomNumber(i);
                }
            else
                {
                this.$planetDensityList[i] = 0.5 + 0.75 * (planetRadius - 15000)/2500 + 0.25 * system.scrambledPseudoRandomNumber(i);
                }
            }
        planetDensity =  this.$planetDensityList[i];
        this.$planetMassList[i] = planetDensity/this.$planetDensity * Math.pow((planetRadius/this.$planetRadius),3);
        this.$escapeVelocityList[i] = this.$planetEscapeVelocity * planetRadius/this.$planetRadius * Math.sqrt(planetDensity/this.$planetDensity);
        escapeVelocity = this.$escapeVelocityList[i];
        this.$planetGravityList[i] = escapeVelocity * escapeVelocity / (planetRadius * 1000) / (9.81*2);
        surfaceGravity =  this.$planetGravityList[i];
        this.$atmosphereMassList[i] = w.$astroLib_atmosphereMass(planetRadius, escapeVelocity, equilibriumTemperature);
        if(planetRadius < 10000)
            {
            atmosphereMass = this.$atmosphereMassList[i];
            this.$atmospherePressureList[i] = atmosphereMass * surfaceGravity;
            }
        else
            {
            this.$atmospherePressureList[i] = 1;
            atmosphereMass = 1 / surfaceGravity;
            }
        this.$atmosphereTemperatureList[i] = w.$astroLib_atmosphereTemperature(equilibriumTemperature, atmosphereMass);
        }
    this.$paintPlanet();
    }

this.$planetInfo = function()
    {
    var planetRadius_scaled;
    var planetMass_scaled;
    var escapeVelocity_scaled;
    var surfaceGravity_scaled;
    var atmospherePressure_scaled;
    for (let i=0; i<system.planets.length; i++)
        {
        if(this.$planetFlag == system.planets[i].radius)
            {
            this.$reportFlag = i;
            }
        }
    var r = this.$reportFlag;
    if(system.planets[r].radius < 75000)
        {
        planetRadius_scaled = system.planets[r].radius / 10;          // default planet scale
        }
    else
        {
        if(system.planets[r].radius >= 75000 && system.planets[r].radius < 100000)
            {
            planetRadius_scaled = (75000 + (system.planets[r].radius - 75000) * 2) / 10;    // super-earth - 2x increment
            }
        else
            {
            if(system.planets[r].radius >= 100000 && system.planets[r].radius < 125000)
                {
                planetRadius_scaled = (150000 + (system.planets[r].radius - 100000) * 4) / 10;    // ice giant - 4x increment
                }
            else
                {
                if(system.planets[r].radius >= 125000)
                    {
                    planetRadius_scaled = (250000 + (system.planets[r].radius - 125000) * 10) / 10;     // gas giant - 10x increment
                    }
                }
            }
        }
    var scale_factor = planetRadius_scaled / system.planets[r].radius * 10;
    planetMass_scaled = this.$planetMassList[r] * Math.pow(scale_factor,3);
    escapeVelocity_scaled = this.$escapeVelocityList[r]/1000 * scale_factor;
    surfaceGravity_scaled = this.$planetGravityList[r] * scale_factor;
    atmospherePressure_scaled = this.$atmospherePressureList[r] * scale_factor;
    if(system.planets[r].radius >= 100000)
        {
        atmospherePressure_scaled = 1;
        }
    if (this.$atmospherePressureList[r] < 0.001)
        {
        atmospherePressure_scaled = 0;
        }
    var planetInfo = "ORBITAL DATA" + "\n"
    + "Orbit radius: " +  this.$orbitRadiusList[r].toFixed(3) + " OU" + "\n"
    + "Orbital period: " + this.$orbitPeriodList[r].toFixed(3) + " days" + "\n"
    + "PHYSICAL DATA" + "\n"
    + "Radius: " + planetRadius_scaled + " km" + "\n"
    + "Mean density: " + this.$planetDensityList[r].toFixed(3) + " g/cm3" + "\n"
    + "Mass: " + planetMass_scaled.toFixed(3) + " TU" + "\n"
    + "Escape velocity: " + escapeVelocity_scaled.toFixed(3) + " km/s" + "\n"
    + "Surface gravity: " + surfaceGravity_scaled.toFixed(3) + " G" + "\n"
    + "Atmosphere pressure: " + atmospherePressure_scaled.toPrecision(4) + " bar" + "\n"
    + "Surface temperature: " + this.$atmosphereTemperatureList[r].toFixed(1) + " K" + "\n";
    mission.runScreen({
        title: "Planet Data Sheet",
//        background: {name: "specialCargoLoad.png", height: 480 },
        message: planetInfo,}
        );
    }

this.$moonInfo = function()
    {
    var w = worldScripts.SpawnMoons;
    for (let i=0; i<system.planets.length; i++)
        {
        if (this.$planetFlag == system.planets[i].radius)
            {
            this.$reportFlag = i;
            }
        }
    var r = this.$reportFlag;
    var moonRadius = system.planets[r].radius;
    var moonOrbitRadius;
    var moonOrbitPeriod;
    var p = w.$targetPlanet_Jovian;
    var moonOrbitVelocity;

    if (moonRadius >= 12500 && moonRadius < 15000)
        {
        moonOrbitRadius = w.$orbitRadius_Luna;
        moonOrbitPeriod = w.$orbitPeriod_Luna / 1440;
        }

    if (moonRadius >= 15000 && moonRadius < 17500)
        {
        moonOrbitRadius = w.$orbitRadius_Selene;
        moonOrbitPeriod = w.$orbitPeriod_Selene / 1440;
        }
    if(moonRadius >= 17500 && moonRadius < 20000)
        {
        moonOrbitRadius = w.$orbitRadius_ZeusInner;
        moonOrbitVelocity = Math.sqrt(this.$escapeVelocityList[p]*this.$escapeVelocityList[p]/moonOrbitRadius/2);
        moonOrbitPeriod = 2.0 * Math.PI * (moonOrbitRadius * system.planets[p].radius * 100 / moonOrbitVelocity) / 86400;
        }
    if(moonRadius >= 20000 && moonRadius < 25000)
        {
        moonOrbitRadius = w.$orbitRadius_ZeusOuter;
        moonOrbitVelocity = Math.sqrt(this.$escapeVelocityList[p]*this.$escapeVelocityList[p]/moonOrbitRadius/2);
        moonOrbitPeriod = 2.0 * Math.PI * (moonOrbitRadius * system.planets[p].radius * 100 / moonOrbitVelocity) / 86400;
        }
    //

    var moonInfo = "ORBITAL DATA" + "\n"
    + "Orbit radius: " +  moonOrbitRadius.toFixed(3) + " R" + "\n"
    + "Orbital period: " + moonOrbitPeriod.toFixed(3) + " days" + "\n"
    + "PHYSICAL DATA" + "\n"
    + "Radius: " + system.planets[r].radius/10 + " km" + "\n"
    + "Mean density: " + this.$planetDensityList[r].toFixed(3) + " g/cm3" + "\n"
    + "Mass: " + this.$planetMassList[r].toFixed(2) + " TU" + "\n"
    + "Escape velocity: " + (this.$escapeVelocityList[r]/1000).toFixed(3) + " km/s" + "\n"
    + "Surface gravity: " + this.$planetGravityList[r].toFixed(3) + " G" + "\n"
    + "Atmosphere pressure: " + this.$atmospherePressureList[r].toPrecision(4) + " bar" + "\n"
    + "Surface temperature: " + this.$atmosphereTemperatureList[r].toFixed(1) + " K" + "\n";
    mission.runScreen({
        title: "Moon Data Sheet",
//        background: {name: "specialCargoLoad.png", height: 480 },
        message: moonInfo,}
        );
    }

this.$paintPlanet = function()
    {
    if (system.isInterstellarSpace) return;

    var paintPlanet1 = system.scrambledPseudoRandomNumber(161);
    var paintPlanet2 = system.scrambledPseudoRandomNumber(162);
    var paintPlanet3 = system.scrambledPseudoRandomNumber(163);
    var paintPlanet4 = system.scrambledPseudoRandomNumber(164);
    var paintPlanet5 = system.scrambledPseudoRandomNumber(165);
    var paintPlanet6 = system.scrambledPseudoRandomNumber(166);
    var paintPlanet7 = system.scrambledPseudoRandomNumber(167);
    var paintPlanet8 = system.scrambledPseudoRandomNumber(168);
    var paintPlanet9 = system.scrambledPseudoRandomNumber(169);
    var paintPlanet10 = system.scrambledPseudoRandomNumber(170);
    var paintPlanet11 = system.scrambledPseudoRandomNumber(171);
    var paintPlanet12 = system.scrambledPseudoRandomNumber(172);

    var aeolisShift = Math.floor(5 * paintPlanet1) + 1;
    var aeolisTexture = ('oplanet_Aeolis' + aeolisShift + '.png');          // Set 1 - Cold wind-eroded deserts
    var aphroditeShift = Math.floor(5 * paintPlanet2) + 1;
    var aphroditeTexture = ('oplanet_Aphrodite' + aphroditeShift + '.png'); // Set 2 - Venus class superdense atmosphere with opaque clouds
    var aresShift = Math.floor(5 * paintPlanet3) + 1;
    var aresTexture = ('oplanet_Ares' + aresShift + '.png');                // Set 3 - Mars class cold desert with impact craters
    var boreusShift = Math.floor(5 * paintPlanet4) + 1;
    var boreusTexture = ('oplanet_Boreus' + boreusShift + '.png');          // Set 4 - Snowball Earth
    var cronusShift = Math.floor(5 * paintPlanet5) + 1;
    var cronusTexture = ('oplanet_Cronus' + cronusShift + '.png');          // Set 5 - Saturn class gas giants
    var hadesShift = Math.floor(5 * paintPlanet6) + 1;
    var hadesTexture = ('oplanet_Hades' + hadesShift + '.png');             // Set 6 - Pluto class ice worlds
    var hermesShift = Math.floor(5 * paintPlanet7) + 1;
    var hermesTexture = ('oplanet_Hermes' + hermesShift + '.png');          // Set 7 - Mercury class heavy cratered dry airless deserts
    var phaethonShift = Math.floor(5 * paintPlanet8) + 1;
    var phaethonTexture = ('oplanet_Phaethon' + phaethonShift + '.png');    // Set 8 - Warm Mars without permafrost and polar caps
    var poseidonShift = Math.floor(5 * paintPlanet9) + 1;
    var poseidonTexture = ('oplanet_Poseidon' + poseidonShift + '.png');    // Set 9 - Neptune class ice giants
    var setShift = Math.floor(5 * paintPlanet10) + 1;
    var setTexture = ('oplanet_Set' + setShift + '.png');                   // Set 10 - Warm sandy deserts
    var typhonShift = Math.floor(5 * paintPlanet11) + 1;
    var typhonTexture = ('oplanet_Typhon' + typhonShift + '.png');          // Set 11 - Hot volcanic worlds
    var zeusShift = Math.floor(5 * paintPlanet12) + 1;
    var zeusTexture = ('oplanet_Zeus' + zeusShift + '.png');                // Set 12 - Jupiter class gas giant

    var w = worldScripts.AstroLibrary;
    var planetOrbitScan = w.$astroLib_orbitRadius();
    var surfaceTemp;
    var surfaceAirPressure;
    var textureIndex;
    var planetTexture;
    for (let i=0; i<system.planets.length; i++) 
        {
        surfaceTemp = this.$atmosphereTemperatureList[i];
        surfaceAirPressure = this.$atmospherePressureList[i];
        // Mercury analog
        if(system.planets[i].radius >= 27500 && system.planets[i].radius < 30000 && system.planets[i] != system.mainPlanet)
            {
            system.planets[i].texture = hermesTexture;
            }  
        // Snowball Earth OR Venus analog
        if(system.planets[i].radius >= 50000 && system.planets[i].radius < 75000 && system.planets[i] != system.mainPlanet)
            {
            if(surfaceAirPressure < 0.01)
                {
                system.planets[i].texture = hermesTexture;
                }
            else
                {
                if(surfaceAirPressure >= 2.5)
                    {
                    system.planets[i].texture = aphroditeTexture;
                    }
                else
                    {
                    textureIndex = this.$climateIndex(surfaceTemp, surfaceAirPressure);
                    switch (textureIndex)
                        {
                        case 2: planetTexture = aeolisTexture; break;
                        case 3: planetTexture = aeolisTexture; break;
                        case 4: planetTexture = setTexture; break;
                        case 20: planetTexture = boreusTexture; break;
                        case 30: planetTexture = aeolisTexture; break;
                        case 40: planetTexture = typhonTexture; break;
                        default: planetTexture = aeolisTexture;
                        }
                    system.planets[i].texture = planetTexture;
                    }
                }
            }
        // Mars analog
        if(system.planets[i].radius >= 30000 && system.planets[i].radius < 50000 && system.planets[i] != system.mainPlanet)
            {            
            if(surfaceAirPressure < 0.01)
                {
                system.planets[i].texture = hermesTexture;
                }
            else
                {
                if(surfaceAirPressure >= 2.5)
                    {
                    system.planets[i].texture = aphroditeTexture;
                    }
                else
                    {
                    textureIndex = this.$climateIndex(surfaceTemp, surfaceAirPressure);
                    switch (textureIndex)
                        {
                        case 2: planetTexture = aresTexture; break;
                        case 3: planetTexture = phaethonTexture; break;
                        case 4: planetTexture = setTexture; break;
                        case 20: planetTexture = aeolisTexture; break;
                        case 30: planetTexture = aeolisTexture; break;
                        case 40: planetTexture = setTexture; break;
                        default: planetTexture = aresTexture;
                        }
                    system.planets[i].texture = planetTexture;
                    }
                }
            }
        // Super Earth
        if(system.planets[i].radius >= 75000 && system.planets[i].radius < 100000 && system.planets[i] != system.mainPlanet)
            {
            system.planets[i].texture = aphroditeTexture;
            }
        // Jupiter analog
        if(system.planets[i].radius >= 150000 && system.planets[i].radius < 175000 && system.planets[i] != system.mainPlanet)
            {
            system.planets[i].texture = zeusTexture;
            } 
        // Saturn analog
        if(system.planets[i].radius >= 125000 && system.planets[i].radius < 150000 && system.planets[i] != system.mainPlanet)
            {
            system.planets[i].texture = cronusTexture;
            } 
        // Neptune analog
        if(system.planets[i].radius >= 100000 && system.planets[i].radius < 125000 && system.planets[i] != system.mainPlanet)
            {
            system.planets[i].texture = poseidonTexture;
            }
        // Pluto analog
        if(system.planets[i].radius >= 25000 && system.planets[i].radius < 27500 && system.planets[i] != system.mainPlanet)
            {
            var orbitHadejan = planetOrbitScan[i];
            if(orbitHadejan > 1)
                {
                system.planets[i].texture = hadesTexture;
                }
            else
                {
                system.planets[i].texture = hermesTexture;
                }
            }
        }
    }

this.$climateIndex = function(surfaceTemp, surfaceAirPressure)
    {
    var tempIndex;
    var pressureIndex;
    var climateIndex;
    if(surfaceTemp < 273)
        {
        tempIndex = 2;
        }
    else
        {
        if(surfaceTemp >= 373)
            {
            tempIndex = 4;
            }
        else
            {
            tempIndex = 3;
            }
        }
    if(surfaceAirPressure < 0.25)
        {
        pressureIndex = 1;
        }
    else
        {
        pressureIndex = 10;
        }
    climateIndex = tempIndex * pressureIndex;
    return climateIndex;
    }